import QtQuick
import QtQuick3D
import QtQuick3D.Helpers
import QtQuick.Controls

import org.qfield
import org.qgis
import Theme

import "qrc:/qml" as QFieldItems

Item {
  id: plugin

  property var mainWindow: iface.mainWindow()

  Component.onCompleted: {
    iface.addItemToPluginsToolbar(quick3DButton)
  }

  QfToolButton {
    id: quick3DButton
    bgcolor: Theme.darkGray
    iconSource: './icon.svg'
    iconColor: Theme.mainColor
    round: true

    onClicked: {
      window.visible = true
    }
  }

  Window {
    id: window
    parent: mainWindow.contentItem
    width: mainWindow.contentItem.width
    height: mainWindow.contentItem.height
    visible: false
    z: 20000

    View3D {
      anchors.fill: parent
      environment: SceneEnvironment {
        backgroundMode: SceneEnvironment.Color
        clearColor: "black"
        InfiniteGrid {
          gridInterval: 100
        }
      }

      Node {
        id: originNode
        eulerRotation.x: -25

        PerspectiveCamera {
          id: cameraNode
          z: 300
        }
      }

      OrbitCameraController {
        origin: originNode
        camera: cameraNode
      }

      OriginGizmo {
        targetNode: cameraNode
        anchors.top: parent.top
        anchors.right: parent.right
        width: 100
        height: 100
      }

      Text {
        anchors.bottom: parent.bottom
        anchors.horizontalCenter: parent.horizontalCenter
        color: "white"
        text: "Click and drag to rotate, Cmd/Alt + click and drag to pan, Scroll-Wheel/Pinch to zoom"
      }
    }

    Rectangle {
      id: originMarker
      width: 50
      height: width
      radius: width * 0.5
      color: "yellow"
      visible: false

      function updateState() {
        const scenePosition = cameraNode.mapToViewport(originNode.scenePosition)
        originMarker.x = scenePosition.x * window.width - originMarker.radius
        originMarker.y = scenePosition.y * window.height - originMarker.radius
        originMarker.visible = scenePosition.z > 0
      }

      Connections {
        target: cameraNode
        function onSceneTransformChanged() {
          originMarker.updateState()
        }
      }
      Connections {
        target: window
        function onWidthChanged() {
          originMarker.updateState()
        }
        function onHeightChanged() {
          originMarker.updateState()
        }
      }
    }

    QfToolButton {
      id: back
      bgcolor: Theme.darkGray
      iconSource: './icon.svg'
      iconColor: Theme.mainColor
      round: true

      onClicked: {
        window.visible = false
      }
    }
  }
}
